@@ -25,14 +25,14 @@ import time
 from openstack.common import log
 from os_collect_config import cache
 from os_collect_config import cfn
-from os_collect_config import common
 from os_collect_config import ec2
 from os_collect_config import exc
+from os_collect_config import heat
 from os_collect_config import heat_local
 from os_collect_config import version
 from oslo.config import cfg
 
-DEFAULT_COLLECTORS = ['heat_local', 'ec2', 'cfn']
+DEFAULT_COLLECTORS = ['heat_local', 'ec2', 'cfn', 'heat']
 opts = [
     cfg.StrOpt('command', short='c',
                help='Command to run on metadata changes. If specified,'
@@ -79,6 +79,7 @@ logger = log.getLogger('os-collect-config')
 
 COLLECTORS = {ec2.name: ec2,
               cfn.name: cfn,
+              heat.name: heat,
               heat_local.name: heat_local}
 
 
@@ -92,18 +93,24 @@ def setup_conf():
     heat_local_group = cfg.OptGroup(name='heat_local',
                                     title='Heat Local Metadata options')
 
+    heat_group = cfg.OptGroup(name='heat',
+                              title='Heat Metadata options')
+
     CONF.register_group(ec2_group)
     CONF.register_group(cfn_group)
     CONF.register_group(heat_local_group)
+    CONF.register_group(heat_group)
     CONF.register_cli_opts(ec2.opts, group='ec2')
     CONF.register_cli_opts(cfn.opts, group='cfn')
     CONF.register_cli_opts(heat_local.opts, group='heat_local')
+    CONF.register_cli_opts(heat.opts, group='heat')
 
     CONF.register_cli_opts(opts)
 
 
-def collect_all(collectors, store=False, requests_impl_map=None):
-    any_changed = False
+def collect_all(collectors, store=False, collector_kwargs_map=None):
+    changed_keys = set()
+    all_keys = list()
     if store:
         paths_or_content = []
     else:
@@ -111,33 +118,34 @@ def collect_all(collectors, store=False, requests_impl_map=None):
 
     for collector in collectors:
         module = COLLECTORS[collector]
-        if requests_impl_map and collector in requests_impl_map:
-            requests_impl = requests_impl_map[collector]
+        if collector_kwargs_map and collector in collector_kwargs_map:
+            collector_kwargs = collector_kwargs_map[collector]
         else:
-            requests_impl = common.requests
+            collector_kwargs = {}
 
         try:
-            content = module.Collector(
-                requests_impl=requests_impl).collect()
+            content = module.Collector(**collector_kwargs).collect()
         except exc.SourceNotAvailable:
             logger.warn('Source [%s] Unavailable.' % collector)
             continue
 
         if store:
             for output_key, output_content in content:
+                all_keys.append(output_key)
                 (changed, path) = cache.store(output_key, output_content)
-                any_changed |= changed
+                if changed:
+                    changed_keys.add(output_key)
                 paths_or_content.append(path)
         else:
             paths_or_content.update(content)
 
-    if any_changed:
-        cache.store_meta_list('os_config_files', collectors)
+    if changed_keys:
+        cache.store_meta_list('os_config_files', all_keys)
         if os.path.exists(CONF.backup_cachedir):
             shutil.rmtree(CONF.backup_cachedir)
         if os.path.exists(CONF.cachedir):
             shutil.copytree(CONF.cachedir, CONF.backup_cachedir)
-    return (any_changed, paths_or_content)
+    return (changed_keys, paths_or_content)
 
 
 def reexec_self(signal=None, frame=None):
@@ -175,7 +183,7 @@ def getfilehash(files):
     return m.hexdigest()
 
 
-def __main__(args=sys.argv, requests_impl_map=None):
+def __main__(args=sys.argv, collector_kwargs_map=None):
     signal.signal(signal.SIGHUP, reexec_self)
     setup_conf()
     CONF(args=args[1:], prog="os-collect-config",
@@ -204,12 +212,12 @@ def __main__(args=sys.argv, requests_impl_map=None):
     config_hash = getfilehash(config_files)
     while True:
         store_and_run = bool(CONF.command and not CONF.print_only)
-        (any_changed, content) = collect_all(
+        (changed_keys, content) = collect_all(
             cfg.CONF.collectors,
             store=store_and_run,
-            requests_impl_map=requests_impl_map)
+            collector_kwargs_map=collector_kwargs_map)
         if store_and_run:
-            if any_changed or CONF.force:
+            if changed_keys or CONF.force:
                 # ignore HUP now since we will reexec after commit anyway
                 signal.signal(signal.SIGHUP, signal.SIG_IGN)
                 try:
@@ -232,8 +240,8 @@ def __main__(args=sys.argv, requests_impl_map=None):
                             logger.warn('Config changed, re-execing now')
                             config_hash = new_config_hash
                 else:
-                    for collector in cfg.CONF.collectors:
-                        cache.commit(collector)
+                    for changed in changed_keys:
+                        cache.commit(changed)
                 if not CONF.one_time:
                     reexec_self()
             else:
